//
//	VERSION MPLAB IDE V.8.63 Y COMPILADOR C18 LITE V3.40
//
//	Moises Melendez 	sesowm@hotmail.com
//	
//	Cdigo fuente de aplicacin del sensor HC-SR04 con el sistema BOLT
//
//	El dispositivo HC-SR04 es un sensor de distancia ultrasnico que permite
//	medir la distancia en un rango desde 2 cm hasta 4 metros, aunque es posible
// 	medir distancias de hasta 4.5 metros el fabricante recomienda no utlizarlo
//	para distancias mayores a los 4 m.
//
//	Es recomendable que la superficie del objeto al que se debe medir la
//	distancia sea lo ms plana posible, es probable que superficies irregulares
//	causen lecturas errneas o materiales suaves como algodn.
//
//	Se sugiere que el mdulo no se conecte directamente a la alimentacin
//	elctrica, si debe conctarse a la alimentacin elctrica despus que 
//	inicie la aplicacin del microcontrolador se deber conectar primero 
//	la terminal GND si no se hace as afectar el trabajo normal del mdulo.
//
//	Para que la medicin de la distancia sea confiable en el rango de trabajo
//	de 2 cm a 4 m, el rea debe ser de al menos de 0.5 metros cuadrados 
//	con una superficie los ms plana posible.
//
//	Principio de funcionamiento
//
//		- Se debe enviar un pulso de disparo por el pin TRIGGER del sensor
//	      de al menos 10 microsegundos.
//		- El sensor enviar 8 pulsos de alta frecuencia por el emisor ultrasonico.
//		- Cuando el receptor ultrasonico recibe la seal de retorno la seal ECHO
//		  se coloca en un nivel alto.
//		- La duracin del pulso en el pin ECHO es directamente proporcional a la
//		  distancia medida por dos ya que debe considerarse el tiempo de retorno 
//		  de la seal desde el objeto hasta el sensor, mediante la siguiente
//		  expresin:
//		
//			d = v * t / 2					...(1)
//
//		La velocidad del sonido en el aire a 25 C es 340 [m/s]
//
//		Por lo tanto la distancia en centmetros en funcin de tiempo medido en 
//		microsegundos es:
//
//			d [cm] = ( 1 / 58 ) * t [us]		...(2)
//
//						 ___
//			   			|   |
//			TRIGGER_____|   |_____________________________________
//
//                           |||||
//			INTERNAL_________|||||________________________________
//
//									 ______________________
//									|  Tiempo quivalente   |
//			ECHO____________________|a la distancia medida |______
//
//
//	CONEXIONES
//
//				SENSOR							  PINES J6
//		______________________		____________________________________					
//
//									- RA4
//		 ___________________		- RA5
//		|   __	   __	VCC |-	 	- RC0 (TRIGGER)  
//		|  /  \	  /  \	TRIG|-		- RC1 (ECHO)		SISTEMA BOLT
//		|  \__/	  \__/	ECHO|-		- VCC 
//		|_______________GND_|-		- GND 
//
//	   _______________________		____________________________________
//
//
//	Se configura el TIMER0 en modo de 16 bits con un preescalador de 4, con la 
//	fuente de reloj interna, as cada 333.33 ns se incrementa el contador, por
//	lo que para obtener el tiempo en microsegundos en funcin de los incrementos 
//	del contador(ticks) tenemos:
//
//		t [us] = ticks * (333.33 ns / 1 tick ) * (1 us / 1000 ns)
//
//		t [us] = 0.33333 * ticks 		...(3)
//
//	Por lo tanto al aplicar (3) en (2):
//
//		d [cm] = (1 / 58) * (0.333333) ticks
//
//		d [cm] = 0.0057 * ticks			...(4)
//
 
#include <p18cxxx.h>
#include <delays.h>
#include <string.h>
#include <stdio.h>		//librera sprintf()
#include <stdlib.h>		//libreras atoi(),atof()
#include <timers.h>
#include "18F2550BOLT.h"
 
void temporizador(void);

extern void _startup( void ); // See c018i.c in your C18 compiler dir 
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800 
void _reset( void ) 
{ 
    _asm goto _startup _endasm 
} 

#pragma code prioridad_nomal = 0x000808
void interrupcion_normal (void)
{
	_asm goto temporizador _endasm
}

#pragma code

char flag=0;

void writeLCDmsg(const far rom char * message);

#pragma interrupt temporizador
void temporizador(void)
{
		//T0CON&=0x7F;			// Deshabilita el contador
		if(flag==0){
			writeLCDmsg((const far rom char*)" Conecta sensor ");
			PORTCbits.RC1=1;
		}
		else{
			writeLCDmsg((const far rom char*)" Fuera de rango ");
			PORTCbits.RC1=0;
		}
		delay_ms(800);
		WriteTimer0(0);
		INTCONbits.TMR0IF = 0;
}

void main(void)
{
	char ent[5];
	char fraq[3];
	char msg1[]="Distan: ";
	char msg2[]=" cm  ";

	int entero, decimal;
	float dist;
	init_bolt();
	InitLCD();

	TRISC &= 0xFE;			// PORTC.0 (TRIGGER) salida, PORTC.1 (ECHO) entrada
	PORTCbits.RC0=0;		//
	T0CON = 0x01;			// Preescalador a 4

    RCON = 0x00;			//IPEN = 0
	INTCON = 0xA0;			//configuramos la interrupcion del timer0	

	delay_ms(200);	

    while (1) 
	{
		WriteTimer0(0);

		T0CON|=0x80;		// Habilita en contador Timer0
		flag=0;

		PORTCbits.RC0=1;		// trigger HIGH
		delay_us(20);			//
		PORTCbits.RC0=0;		// Pulso de disparo

		while(!PORTCbits.RC1);	// Espera el retorno de la seal

		WriteTimer0(0);
		flag=1;
 
		T0CON|=0x80;			// Habilita en contador Timer0

		while(PORTCbits.RC1);	// Espera a que la seal ECHO baje

		T0CON&=0x7F;			// Deshabilita el contador
		dist= (333.33/58000)*ReadTimer0();

		if(dist<2.0 || dist>400.0) writeLCDmsg((const far rom char*)" Fuera de rango ");
		else{
			entero=dist;
		    decimal=(dist-entero)*10;	// 1 digito decimal
			sprintf(ent,(const far rom char*)"%d",entero);		//convierte a una cadena
			sprintf(fraq,(const far rom char*)"%01d",decimal);
			
			ClearScreen();
			PrintString(msg1);
			GotoXY(1,0);
			PrintString(ent);		
			PutLCD('.');
			PrintString(fraq);
			PrintString(msg2);
		}
		delay_ms(800);
    } 
} 

//
//	Funcin que enva una cadena de caracteres en ROM al LCD
//	Recibe la referencia de la cadena y enva cada uno de los caracteres
//	que componen la cadena en ROM, no retorna valor.
//
void writeLCDmsg(const far rom char * message)
{
	unsigned char i;
	ClearScreen();							// Clear screen
	GotoXY(0,0);							
	for(i=0; i<8; i++) PutLCD(message[i]);	// Writes bytes to serial port
	GotoXY(1,0);
	for(i=8; i<strlenpgm(message);i++) PutLCD(message[i]);
}